@@ -1,7 +1,7 @@
 // SPDX-FileCopyrightText: 2021 Heiko Schaefer <heiko@schaefer.name>
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-use crate::errors::{OcErrorStatus, OpenpgpCardError};
+use crate::{Error, StatusBytes};
 use std::convert::TryFrom;
 
 /// Response from the card to a command.
@@ -24,40 +24,37 @@ impl Response {
 /// "Raw" APDU Response, including the status bytes.
 ///
 /// This type is used for processing inside the openpgp-card crate
-/// (raw responses with a non-ok status sometimes need to be processed e.g.
-/// when a response is sent from the card in "chained" format).
+/// (raw responses with a non-ok status sometimes need to be processed,
+/// e.g. when a card sends a response in "chained" format).
 #[allow(unused)]
 #[derive(Clone, Debug)]
 pub(crate) struct RawResponse {
     data: Vec<u8>,
-    sw1: u8,
-    sw2: u8,
+    status: StatusBytes,
 }
 
 impl TryFrom<RawResponse> for Response {
-    type Error = OpenpgpCardError;
+    type Error = Error;
 
     fn try_from(value: RawResponse) -> Result<Self, Self::Error> {
         if value.is_ok() {
             Ok(Response { data: value.data })
         } else {
-            Err(OpenpgpCardError::OcStatus(OcErrorStatus::from(
-                value.status(),
-            )))
+            Err(value.status().into())
         }
     }
 }
 
 impl RawResponse {
-    pub fn check_ok(&self) -> Result<(), OcErrorStatus> {
+    pub fn check_ok(&self) -> Result<(), StatusBytes> {
         if !self.is_ok() {
-            Err(OcErrorStatus::from((self.sw1, self.sw2)))
+            Err(self.status())
         } else {
             Ok(())
         }
     }
 
-    pub fn data(&self) -> Result<&[u8], OcErrorStatus> {
+    pub fn data(&self) -> Result<&[u8], StatusBytes> {
         self.check_ok()?;
         Ok(&self.data)
     }
@@ -71,33 +68,34 @@ impl RawResponse {
         &mut self.data
     }
 
-    pub(crate) fn set_status(&mut self, new_status: (u8, u8)) {
-        self.sw1 = new_status.0;
-        self.sw2 = new_status.1;
+    pub(crate) fn set_status(&mut self, new_status: StatusBytes) {
+        self.status = new_status;
     }
 
-    pub fn status(&self) -> (u8, u8) {
-        (self.sw1, self.sw2)
+    pub fn status(&self) -> StatusBytes {
+        self.status
     }
 
     /// Is the response status "ok"? (0x90, 0x00)
     pub fn is_ok(&self) -> bool {
-        self.status() == (0x90, 0x00)
+        self.status() == StatusBytes::Ok
     }
 }
 
 impl TryFrom<Vec<u8>> for RawResponse {
-    type Error = OpenpgpCardError;
+    type Error = Error;
 
     fn try_from(mut data: Vec<u8>) -> Result<Self, Self::Error> {
         let sw2 = data
             .pop()
-            .ok_or_else(|| OpenpgpCardError::ResponseLength(data.len()))?;
+            .ok_or_else(|| Error::ResponseLength(data.len()))?;
         let sw1 = data
             .pop()
-            .ok_or_else(|| OpenpgpCardError::ResponseLength(data.len()))?;
+            .ok_or_else(|| Error::ResponseLength(data.len()))?;
 
-        Ok(RawResponse { data, sw1, sw2 })
+        let status = (sw1, sw2).into();
+
+        Ok(RawResponse { data, status })
     }
 }
 
