@@ -1,29 +1,30 @@
 // SPDX-FileCopyrightText: 2021 Heiko Schaefer <heiko@schaefer.name>
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
+//! CardApp exposes functionality of the "OpenPGP card" application.
+
 use std::borrow::BorrowMut;
 use std::convert::TryFrom;
 use std::convert::TryInto;
-use std::time::SystemTime;
 
 use anyhow::{anyhow, Result};
 
-use crate::algorithm::{Algo, AlgoInfo, AlgoSimple, RsaAttrs};
+use crate::algorithm::{Algo, AlgoInfo, AlgoSimple};
 use crate::apdu::{commands, response::Response};
 use crate::card_do::{
-    ApplicationRelatedData, Cardholder, Fingerprint, PWStatus,
-    SecuritySupportTemplate, Sex,
+    ApplicationRelatedData, CardholderRelatedData, Fingerprint,
+    KeyGenerationTime, PWStatusBytes, SecuritySupportTemplate, Sex,
 };
 use crate::crypto_data::{
-    CardUploadableKey, Cryptogram, EccType, Hash, PublicKeyMaterial,
+    CardUploadableKey, Cryptogram, Hash, PublicKeyMaterial,
 };
-use crate::errors::OpenpgpCardError;
-use crate::tlv::{tag::Tag, Tlv, TlvEntry};
+use crate::tlv::{tag::Tag, value::Value, Tlv};
 use crate::{apdu, keys, CardCaps, CardClientBox, KeyType};
+use crate::{Error, StatusBytes};
 
 /// Low-level access to OpenPGP card functionality.
 ///
-/// No checks are performed here (e.g. for valid data lengths).
+/// Not many checks are performed here (e.g. for valid data lengths).
 /// Such checks should be performed on a higher layer, if needed.
 ///
 /// Also, no caching of data is done here. If necessary, caching should
@@ -32,21 +33,28 @@ pub struct CardApp {
     card_client: CardClientBox,
 }
 
-impl CardApp {
-    /// Create a CardApp object based on a [`CardClientBox`].
-    pub fn new(card_client: CardClientBox) -> Self {
+impl From<CardClientBox> for CardApp {
+    fn from(card_client: CardClientBox) -> Self {
         Self { card_client }
     }
+}
 
-    /// Take the CardClientBox out of a CardApp
-    pub fn take_card(self) -> CardClientBox {
-        self.card_client
+impl From<CardApp> for CardClientBox {
+    fn from(card_app: CardApp) -> CardClientBox {
+        card_app.card_client
+    }
+}
+
+impl CardApp {
+    /// Get the CardClient for this CardApp
+    pub(crate) fn get_card_client(&mut self) -> &mut CardClientBox {
+        &mut self.card_client
     }
 
-    /// Read capabilities from the card, and set them in the CardApp.
+    /// Initialize the CardCaps settings in the underlying CardClient
+    /// from the data in `ard`.
     ///
-    /// Also initializes the underlying CardClient with the caps - some
-    /// implementations may need this information.
+    /// This should be done at an early point, soon after opening the card.
     pub fn init_caps(&mut self, ard: &ApplicationRelatedData) -> Result<()> {
         // Determine chaining/extended length support from card
         // metadata and cache this information in CardApp (as a
@@ -64,7 +72,7 @@ impl CardApp {
 
         let (max_cmd_bytes, max_rsp_bytes) =
             if let Ok(Some(eli)) = ard.get_extended_length_information() {
-                (eli.max_command_bytes, eli.max_response_bytes)
+                (eli.max_command_bytes(), eli.max_response_bytes())
             } else {
                 (255, 255)
             };
@@ -81,14 +89,10 @@ impl CardApp {
         Ok(())
     }
 
-    pub fn card(&mut self) -> &mut CardClientBox {
-        &mut self.card_client
-    }
-
     // --- select ---
 
-    /// "Select" the OpenPGP card application
-    pub fn select(&mut self) -> Result<Response, OpenpgpCardError> {
+    /// Select the OpenPGP card application
+    pub fn select(&mut self) -> Result<Response, Error> {
         let select_openpgp = commands::select_openpgp();
         apdu::send_command(&mut self.card_client, select_openpgp, false)?
             .try_into()
@@ -96,24 +100,28 @@ impl CardApp {
 
     // --- get data ---
 
-    /// Load "application related data".
+    /// Get the "application related data" from the card.
     ///
-    /// This data should probably be cached in a higher layer. Some parts of
-    /// it are needed regularly, and it will not usually change during
-    /// normal use of a card.
-    pub fn get_app_data(&mut self) -> Result<ApplicationRelatedData> {
+    /// (This data should probably be cached in a higher layer. Some parts of
+    /// it are needed regularly, and it does not usually change during
+    /// normal use of a card.)
+    pub fn get_application_related_data(
+        &mut self,
+    ) -> Result<ApplicationRelatedData> {
         let ad = commands::get_application_data();
         let resp = apdu::send_command(&mut self.card_client, ad, true)?;
-        let entry = TlvEntry::from(resp.data()?, true)?;
+        let value = Value::from(resp.data()?, true)?;
 
-        log::debug!(" App data TlvEntry: {:x?}", entry);
+        log::debug!(" App data Value: {:x?}", value);
 
-        Ok(ApplicationRelatedData(Tlv(Tag::from([0x6E]), entry)))
+        Ok(ApplicationRelatedData(Tlv::new(Tag::from([0x6E]), value)))
     }
 
-    /// Get data from "private use" DO, `num` must be between 1 and 4.
+    /// Get data from "private use" DO.
+    ///
+    /// `num` must be between 1 and 4.
     pub fn get_private(&mut self, num: u8) -> Result<Vec<u8>> {
-        assert!(num >= 1 && num <= 4);
+        assert!((1..=4).contains(&num));
 
         let cmd = commands::get_private_do(num);
         let resp = apdu::send_command(&mut self.card_client, cmd, true)?;
@@ -162,12 +170,14 @@ impl CardApp {
     }
 
     // --- cardholder related data (65) ---
-    pub fn get_cardholder_related_data(&mut self) -> Result<Cardholder> {
+    pub fn get_cardholder_related_data(
+        &mut self,
+    ) -> Result<CardholderRelatedData> {
         let crd = commands::cardholder_related_data();
         let resp = apdu::send_command(&mut self.card_client, crd, true)?;
         resp.check_ok()?;
 
-        Cardholder::try_from(resp.data()?)
+        CardholderRelatedData::try_from(resp.data()?)
     }
 
     // --- security support template (7a) ---
@@ -179,11 +189,11 @@ impl CardApp {
         resp.check_ok()?;
 
         let tlv = Tlv::try_from(resp.data()?)?;
-        let res = tlv
-            .find(&Tag::from([0x93]))
-            .ok_or(anyhow!("Couldn't get SecuritySupportTemplate DO"))?;
+        let res = tlv.find(&[0x93].into()).ok_or_else(|| {
+            anyhow!("Couldn't get SecuritySupportTemplate DO")
+        })?;
 
-        if let TlvEntry::S(data) = res {
+        if let Value::S(data) = res {
             let mut data = data.to_vec();
             assert_eq!(data.len(), 3);
 
@@ -201,15 +211,13 @@ impl CardApp {
     ///
     /// Call select_data() before calling this fn, to select a particular
     /// certificate (if the card supports multiple certificates).
-    pub fn get_cardholder_certificate(
-        &mut self,
-    ) -> Result<Response, OpenpgpCardError> {
+    pub fn get_cardholder_certificate(&mut self) -> Result<Response, Error> {
         let cmd = commands::get_cardholder_certificate();
         apdu::send_command(&mut self.card_client, cmd, true)?.try_into()
     }
 
-    /// DO "Algorithm Information" (0xFA)
-    pub fn list_supported_algo(&mut self) -> Result<Option<AlgoInfo>> {
+    /// DO "Algorithm Information"
+    pub fn get_algo_info(&mut self) -> Result<Option<AlgoInfo>> {
         let resp = apdu::send_command(
             &mut self.card_client,
             commands::get_algo_list(),
@@ -221,14 +229,16 @@ impl CardApp {
         Ok(Some(ai))
     }
 
+    /// SELECT DATA "select a DO in the current template"
+    /// (e.g. for cardholder certificate)
     pub fn select_data(
         &mut self,
         num: u8,
         tag: &[u8],
-    ) -> Result<Response, OpenpgpCardError> {
-        let tlv = Tlv(
-            Tag(vec![0x60]),
-            TlvEntry::C(vec![Tlv(Tag(vec![0x5c]), TlvEntry::S(tag.to_vec()))]),
+    ) -> Result<Response, Error> {
+        let tlv = Tlv::new(
+            [0x60],
+            Value::C(vec![Tlv::new([0x5c], Value::S(tag.to_vec()))]),
         );
 
         let data = tlv.serialize();
@@ -239,7 +249,14 @@ impl CardApp {
 
     // ----------
 
-    /// Delete all state on this OpenPGP card
+    /// Reset all state on this OpenPGP card.
+    ///
+    /// Note: the "factory reset" operation is not directly offered by the
+    /// card. It is implemented as a series of OpenPGP card commands:
+    /// - send 4 bad requests to verify pw1
+    /// - send 4 bad requests to verify pw3
+    /// - terminate_df
+    /// - activate_file
     pub fn factory_reset(&mut self) -> Result<()> {
         // send 4 bad requests to verify pw1
         // [apdu 00 20 00 81 08 40 40 40 40 40 40 40 40]
@@ -247,8 +264,8 @@ impl CardApp {
             let verify = commands::verify_pw1_81([0x40; 8].to_vec());
             let resp =
                 apdu::send_command(&mut self.card_client, verify, false)?;
-            if !(resp.status() == (0x69, 0x82)
-                || resp.status() == (0x69, 0x83))
+            if !(resp.status() == StatusBytes::SecurityStatusNotSatisfied
+                || resp.status() == StatusBytes::AuthenticationMethodBlocked)
             {
                 return Err(anyhow!("Unexpected status for reset, at pw1."));
             }
@@ -261,8 +278,8 @@ impl CardApp {
             let resp =
                 apdu::send_command(&mut self.card_client, verify, false)?;
 
-            if !(resp.status() == (0x69, 0x82)
-                || resp.status() == (0x69, 0x83))
+            if !(resp.status() == StatusBytes::SecurityStatusNotSatisfied
+                || resp.status() == StatusBytes::AuthenticationMethodBlocked)
             {
                 return Err(anyhow!("Unexpected status for reset, at pw3."));
             }
@@ -278,62 +295,131 @@ impl CardApp {
         let resp = apdu::send_command(&mut self.card_client, act, false)?;
         resp.check_ok()?;
 
-        // FIXME: does the connection need to be re-opened on some cards,
-        // after reset?!
-
         Ok(())
     }
 
+    /// Verify pw1 (user) for signing operation (mode 81) and set an
+    /// appropriate access status.
+    ///
+    /// Depending on the PW1 status byte (see Extended Capabilities) this
+    /// access condition is only valid for one PSO:CDS command or remains
+    /// valid for several attempts.
     pub fn verify_pw1_for_signing(
         &mut self,
         pin: &str,
-    ) -> Result<Response, OpenpgpCardError> {
-        assert!(pin.len() >= 6); // FIXME: Err
-
+    ) -> Result<Response, Error> {
         let verify = commands::verify_pw1_81(pin.as_bytes().to_vec());
         apdu::send_command(&mut self.card_client, verify, false)?.try_into()
     }
 
-    pub fn check_pw1(&mut self) -> Result<Response, OpenpgpCardError> {
-        let verify = commands::verify_pw1_82(vec![]);
+    /// Check the current access of PW1 for signing (mode 81).
+    ///
+    /// If verification is not required, an empty Ok Response is returned.
+    ///
+    /// (Note: some cards don't correctly implement this feature,
+    /// e.g. yubikey 5)
+    pub fn check_pw1_for_signing(&mut self) -> Result<Response, Error> {
+        let verify = commands::verify_pw1_81(vec![]);
         apdu::send_command(&mut self.card_client, verify, false)?.try_into()
     }
 
-    pub fn verify_pw1(
-        &mut self,
-        pin: &str,
-    ) -> Result<Response, OpenpgpCardError> {
-        assert!(pin.len() >= 6); // FIXME: Err
-
+    /// Verify PW1 (user) and set an appropriate access status.
+    /// (For operations except signing, mode 82).
+    pub fn verify_pw1(&mut self, pin: &str) -> Result<Response, Error> {
         let verify = commands::verify_pw1_82(pin.as_bytes().to_vec());
         apdu::send_command(&mut self.card_client, verify, false)?.try_into()
     }
 
-    pub fn check_pw3(&mut self) -> Result<Response, OpenpgpCardError> {
+    /// Check the current access of PW1.
+    /// (For operations except signing, mode 82).
+    ///
+    /// If verification is not required, an empty Ok Response is returned.
+    ///
+    /// (Note: some cards don't correctly implement this feature,
+    /// e.g. yubikey 5)
+    pub fn check_pw1(&mut self) -> Result<Response, Error> {
+        let verify = commands::verify_pw1_82(vec![]);
+        apdu::send_command(&mut self.card_client, verify, false)?.try_into()
+    }
+
+    /// Verify PW3 (admin) and set an appropriate access status.
+    pub fn verify_pw3(&mut self, pin: &str) -> Result<Response, Error> {
+        let verify = commands::verify_pw3(pin.as_bytes().to_vec());
+        apdu::send_command(&mut self.card_client, verify, false)?.try_into()
+    }
+
+    /// Check the current access of PW3 (admin).
+    ///
+    /// If verification is not required, an empty Ok Response is returned.
+    ///
+    /// (Note: some cards don't correctly implement this feature,
+    /// e.g. yubikey 5)
+    pub fn check_pw3(&mut self) -> Result<Response, Error> {
         let verify = commands::verify_pw3(vec![]);
         apdu::send_command(&mut self.card_client, verify, false)?.try_into()
     }
 
-    pub fn verify_pw3(
+    /// Change the value of PW1 (user password).
+    ///
+    /// The current value of PW1 must be presented in `old` for authorization.
+    pub fn change_pw1(
         &mut self,
-        pin: &str,
-    ) -> Result<Response, OpenpgpCardError> {
-        assert!(pin.len() >= 8); // FIXME: Err
+        old: &str,
+        new: &str,
+    ) -> Result<Response, Error> {
+        let mut data = vec![];
+        data.extend(old.as_bytes());
+        data.extend(new.as_bytes());
 
-        let verify = commands::verify_pw3(pin.as_bytes().to_vec());
-        apdu::send_command(&mut self.card_client, verify, false)?.try_into()
+        let change = commands::change_pw1(data);
+        apdu::send_command(&mut self.card_client, change, false)?.try_into()
+    }
+
+    /// Change the value of PW3 (admin password).
+    ///
+    /// The current value of PW3 must be presented in `old` for authorization.
+    pub fn change_pw3(
+        &mut self,
+        old: &str,
+        new: &str,
+    ) -> Result<Response, Error> {
+        let mut data = vec![];
+        data.extend(old.as_bytes());
+        data.extend(new.as_bytes());
+
+        let change = commands::change_pw3(data);
+        apdu::send_command(&mut self.card_client, change, false)?.try_into()
+    }
+
+    /// Reset the error counter for PW1 (user password) and set a new value
+    /// for PW1.
+    ///
+    /// For authorization, either:
+    /// - PW3 must have been verified previously,
+    /// - secure messaging must be currently used,
+    /// - the resetting_code must be presented.
+    pub fn reset_retry_counter_pw1(
+        &mut self,
+        new_pw1: Vec<u8>,
+        resetting_code: Option<Vec<u8>>,
+    ) -> Result<Response, Error> {
+        let reset = commands::reset_retry_counter_pw1(resetting_code, new_pw1);
+        apdu::send_command(&mut self.card_client, reset, false)?.try_into()
     }
 
     // --- decrypt ---
 
     /// Decrypt the ciphertext in `dm`, on the card.
-    pub fn decrypt(
-        &mut self,
-        dm: Cryptogram,
-    ) -> Result<Vec<u8>, OpenpgpCardError> {
+    ///
+    /// (This is a wrapper around the low-level pso_decipher
+    /// operation, it builds the required `data` field from `dm`)
+    pub fn decipher(&mut self, dm: Cryptogram) -> Result<Vec<u8>, Error> {
         match dm {
             Cryptogram::RSA(message) => {
                 let mut data = vec![0x0];
+
+                // FIXME: The spec says we should "format according to PKCS#1"
+
                 data.extend_from_slice(message);
 
                 // Call the card to decrypt `data`
@@ -341,25 +427,22 @@ impl CardApp {
             }
             Cryptogram::ECDH(eph) => {
                 // External Public Key
-                let epk = Tlv(Tag(vec![0x86]), TlvEntry::S(eph.to_vec()));
+                let epk = Tlv::new([0x86], Value::S(eph.to_vec()));
 
                 // Public Key DO
-                let pkdo = Tlv(Tag(vec![0x7f, 0x49]), TlvEntry::C(vec![epk]));
+                let pkdo = Tlv::new([0x7f, 0x49], Value::C(vec![epk]));
 
                 // Cipher DO
-                let cdo = Tlv(Tag(vec![0xa6]), TlvEntry::C(vec![pkdo]));
+                let cdo = Tlv::new([0xa6], Value::C(vec![pkdo]));
 
                 self.pso_decipher(cdo.serialize())
             }
         }
     }
 
-    /// Run decryption operation on the smartcard
+    /// Run decryption operation on the smartcard (low level operation)
     /// (7.2.11 PSO: DECIPHER)
-    pub fn pso_decipher(
-        &mut self,
-        data: Vec<u8>,
-    ) -> Result<Vec<u8>, OpenpgpCardError> {
+    fn pso_decipher(&mut self, data: Vec<u8>) -> Result<Vec<u8>, Error> {
         // The OpenPGP card is already connected and PW1 82 has been verified
         let dec_cmd = commands::decryption(data);
         let resp = apdu::send_command(&mut self.card_client, dec_cmd, true)?;
@@ -371,30 +454,31 @@ impl CardApp {
     // --- sign ---
 
     /// Sign `hash`, on the card.
+    ///
+    /// (This is a wrapper around the low-level
+    /// pso_compute_digital_signature operation.
+    /// It builds the required `data` field from `hash`)
     pub fn signature_for_hash(
         &mut self,
         hash: Hash,
-    ) -> Result<Vec<u8>, OpenpgpCardError> {
+    ) -> Result<Vec<u8>, Error> {
         let data = match hash {
             Hash::SHA256(_) | Hash::SHA384(_) | Hash::SHA512(_) => {
-                let tlv = Tlv(
-                    Tag(vec![0x30]),
-                    TlvEntry::C(vec![
-                        Tlv(
-                            Tag(vec![0x30]),
-                            TlvEntry::C(vec![
-                                Tlv(
-                                    Tag(vec![0x06]),
+                let tlv = Tlv::new(
+                    [0x30],
+                    Value::C(vec![
+                        Tlv::new(
+                            [0x30],
+                            Value::C(vec![
+                                Tlv::new(
+                                    [0x06],
                                     // unwrapping is ok, for SHA*
-                                    TlvEntry::S(hash.oid().unwrap().to_vec()),
+                                    Value::S(hash.oid().unwrap().to_vec()),
                                 ),
-                                Tlv(Tag(vec![0x05]), TlvEntry::S(vec![])),
+                                Tlv::new([0x05], Value::S(vec![])),
                             ]),
                         ),
-                        Tlv(
-                            Tag(vec![0x04]),
-                            TlvEntry::S(hash.digest().to_vec()),
-                        ),
+                        Tlv::new([0x04], Value::S(hash.digest().to_vec())),
                     ]),
                 );
 
@@ -404,15 +488,15 @@ impl CardApp {
             Hash::ECDSA(d) => d.to_vec(),
         };
 
-        self.compute_digital_signature(data)
+        self.pso_compute_digital_signature(data)
     }
 
-    /// Run signing operation on the smartcard
+    /// Run signing operation on the smartcard (low level operation)
     /// (7.2.10 PSO: COMPUTE DIGITAL SIGNATURE)
-    pub fn compute_digital_signature(
+    fn pso_compute_digital_signature(
         &mut self,
         data: Vec<u8>,
-    ) -> Result<Vec<u8>, OpenpgpCardError> {
+    ) -> Result<Vec<u8>, Error> {
         let dec_cmd = commands::signature(data);
 
         let resp = apdu::send_command(&mut self.card_client, dec_cmd, true)?;
@@ -422,12 +506,15 @@ impl CardApp {
 
     // --- admin ---
 
-    /// Set data of "private use" DO, `num` must be between 1 and 4.
+    /// Set data of "private use" DO.
+    ///
+    /// `num` must be between 1 and 4.
+    ///
     /// Access condition:
     /// - 1/3 need PW1 (82)
     /// - 2/4 need PW3
     pub fn set_private(&mut self, num: u8, data: Vec<u8>) -> Result<Vec<u8>> {
-        assert!(num >= 1 && num <= 4);
+        assert!((1..=4).contains(&num));
 
         let cmd = commands::put_private_do(num, data);
         let resp = apdu::send_command(&mut self.card_client, cmd, true)?;
@@ -435,44 +522,36 @@ impl CardApp {
         Ok(resp.data()?.to_vec())
     }
 
-    pub fn set_name(
-        &mut self,
-        name: &str,
-    ) -> Result<Response, OpenpgpCardError> {
+    pub fn set_name(&mut self, name: &str) -> Result<Response, Error> {
         let put_name = commands::put_name(name.as_bytes().to_vec());
         apdu::send_command(&mut self.card_client, put_name, false)?.try_into()
     }
 
-    pub fn set_lang(
-        &mut self,
-        lang: &str,
-    ) -> Result<Response, OpenpgpCardError> {
+    pub fn set_lang(&mut self, lang: &str) -> Result<Response, Error> {
         let put_lang = commands::put_lang(lang.as_bytes().to_vec());
         apdu::send_command(self.card_client.borrow_mut(), put_lang, false)?
             .try_into()
     }
 
-    pub fn set_sex(&mut self, sex: Sex) -> Result<Response, OpenpgpCardError> {
+    pub fn set_sex(&mut self, sex: Sex) -> Result<Response, Error> {
         let put_sex = commands::put_sex((&sex).into());
         apdu::send_command(self.card_client.borrow_mut(), put_sex, false)?
             .try_into()
     }
 
-    pub fn set_url(
-        &mut self,
-        url: &str,
-    ) -> Result<Response, OpenpgpCardError> {
+    pub fn set_url(&mut self, url: &str) -> Result<Response, Error> {
         let put_url = commands::put_url(url.as_bytes().to_vec());
         apdu::send_command(&mut self.card_client, put_url, false)?.try_into()
     }
 
     pub fn set_creation_time(
         &mut self,
-        time: u32,
+        time: KeyGenerationTime,
         key_type: KeyType,
-    ) -> Result<Response, OpenpgpCardError> {
+    ) -> Result<Response, Error> {
         // Timestamp update
         let time_value: Vec<u8> = time
+            .get()
             .to_be_bytes()
             .iter()
             .skip_while(|&&e| e == 0)
@@ -491,13 +570,13 @@ impl CardApp {
         &mut self,
         fp: Fingerprint,
         key_type: KeyType,
-    ) -> Result<Response, OpenpgpCardError> {
+    ) -> Result<Response, Error> {
         let fp_cmd = commands::put_data(
             &[key_type.get_fingerprint_put_tag()],
             fp.as_bytes().to_vec(),
         );
 
-        apdu::send_command(self.card(), fp_cmd, false)?.try_into()
+        apdu::send_command(self.get_card_client(), fp_cmd, false)?.try_into()
     }
 
     /// Set PW Status Bytes.
@@ -513,13 +592,13 @@ impl CardApp {
     /// (See OpenPGP card spec, pg. 28)
     pub fn set_pw_status_bytes(
         &mut self,
-        pw_status: &PWStatus,
+        pw_status: &PWStatusBytes,
         long: bool,
-    ) -> Result<Response, OpenpgpCardError> {
+    ) -> Result<Response, Error> {
         let data = pw_status.serialize_for_put(long);
 
         let cmd = commands::put_pw_status(data);
-        apdu::send_command(self.card(), cmd, false)?.try_into()
+        apdu::send_command(self.get_card_client(), cmd, false)?.try_into()
     }
 
     /// Set cardholder certificate (for AUT, DEC or SIG).
@@ -529,90 +608,55 @@ impl CardApp {
     pub fn set_cardholder_certificate(
         &mut self,
         data: Vec<u8>,
-    ) -> Result<Response, OpenpgpCardError> {
+    ) -> Result<Response, Error> {
         let cmd = commands::put_cardholder_certificate(data);
         apdu::send_command(&mut self.card_client, cmd, false)?.try_into()
     }
 
-    /// Set algorithm attributes [4.4.3.9 Algorithm Attributes]
+    /// Set algorithm attributes
+    /// (4.4.3.9 Algorithm Attributes)
     pub fn set_algorithm_attributes(
         &mut self,
         key_type: KeyType,
         algo: &Algo,
-    ) -> Result<Response, OpenpgpCardError> {
-        // FIXME: caching?
-        let ard = self.get_app_data()?;
-
-        // FIXME: reuse "e" from card, if no algo list is available
-        let _cur_algo = ard.get_algorithm_attributes(key_type)?;
-
-        let data = match algo {
-            Algo::Rsa(rsa) => Self::rsa_algo_attrs(rsa)?,
-            Algo::Ecc(ecc) => Self::ecc_algo_attrs(ecc.oid(), ecc.ecc_type),
-            _ => unimplemented!(),
-        };
-
+    ) -> Result<Response, Error> {
         // Command to PUT the algorithm attributes
-        let cmd = commands::put_data(&[key_type.get_algorithm_tag()], data);
+        let cmd = commands::put_data(
+            &[key_type.get_algorithm_tag()],
+            algo.get_data()?,
+        );
 
         apdu::send_command(&mut self.card_client, cmd, false)?.try_into()
     }
 
-    fn rsa_algo_attrs(algo_attrs: &RsaAttrs) -> Result<Vec<u8>> {
-        // Algorithm ID (01 = RSA (Encrypt or Sign))
-        let mut algo_attributes = vec![0x01];
-
-        // Length of modulus n in bit
-        algo_attributes.extend(&algo_attrs.len_n.to_be_bytes());
-
-        // Length of public exponent e in bit
-        algo_attributes.push(0x00);
-        algo_attributes.push(algo_attrs.len_e as u8);
-
-        // Import-Format of private key
-        // (This fn currently assumes import_format "00 = standard (e, p, q)")
-        if algo_attrs.import_format != 0 {
-            return Err(anyhow!(
-                "Unexpected RSA input format (only 0 is supported)"
-            ));
-        }
-
-        algo_attributes.push(algo_attrs.import_format);
-
-        Ok(algo_attributes)
+    /// Set resetting code
+    /// (4.3.4 Resetting Code)
+    pub fn set_resetting_code(
+        &mut self,
+        resetting_code: Vec<u8>,
+    ) -> Result<Response, Error> {
+        let cmd = commands::put_data(&[0xd3], resetting_code);
+        apdu::send_command(&mut self.card_client, cmd, false)?.try_into()
     }
 
-    fn ecc_algo_attrs(oid: &[u8], ecc_type: EccType) -> Vec<u8> {
-        let algo_id = match ecc_type {
-            EccType::EdDSA => 0x16,
-            EccType::ECDH => 0x12,
-            EccType::ECDSA => 0x13,
-        };
-
-        let mut algo_attributes = vec![algo_id];
-        algo_attributes.extend(oid);
-        // Leave Import-Format unset, for default (pg. 35)
-
-        algo_attributes
-    }
-
-    /// Upload an existing private key to the card.
+    /// Import an existing private key to the card.
     /// (This implicitly sets the algorithm info, fingerprint and timestamp)
-    pub fn upload_key(
+    pub fn key_import(
         &mut self,
         key: Box<dyn CardUploadableKey>,
         key_type: KeyType,
-    ) -> Result<(), OpenpgpCardError> {
-        let algo_list = self.list_supported_algo();
+    ) -> Result<(), Error> {
+        let algo_list = self.get_algo_info();
 
         // An error is ok - it's fine if a card doesn't offer a list of
         // supported algorithms
         let algo_list = algo_list.unwrap_or(None);
 
-        keys::upload_key(self, key, key_type, algo_list)
+        keys::key_import(self, key, key_type, algo_list)
     }
 
     /// Generate a key on the card.
+    /// (7.2.14 GENERATE ASYMMETRIC KEY PAIR)
     ///
     /// If the `algo` parameter is Some, then this algorithm will be set on
     /// the card for "key_type".
@@ -620,35 +664,44 @@ impl CardApp {
         &mut self,
         fp_from_pub: fn(
             &PublicKeyMaterial,
-            SystemTime,
+            KeyGenerationTime,
             KeyType,
-        ) -> Result<Fingerprint, OpenpgpCardError>,
+        ) -> Result<Fingerprint, Error>,
         key_type: KeyType,
         algo: Option<&Algo>,
-    ) -> Result<(PublicKeyMaterial, u32), OpenpgpCardError> {
+    ) -> Result<(PublicKeyMaterial, KeyGenerationTime), Error> {
         keys::gen_key_with_metadata(self, fp_from_pub, key_type, algo)
     }
 
     /// Generate a key on the card.
-    /// Use a simplified algo selector enum.
+    /// (7.2.14 GENERATE ASYMMETRIC KEY PAIR)
+    ///
+    /// This is a convenience wrapper around generate_key() which allows
+    /// using the simplified `AlgoSimple` algorithm selector enum.
     pub fn generate_key_simple(
         &mut self,
         fp_from_pub: fn(
             &PublicKeyMaterial,
-            SystemTime,
+            KeyGenerationTime,
             KeyType,
-        ) -> Result<Fingerprint, OpenpgpCardError>,
+        ) -> Result<Fingerprint, Error>,
         key_type: KeyType,
         algo: AlgoSimple,
-    ) -> Result<(PublicKeyMaterial, u32), OpenpgpCardError> {
-        let algo = algo.to_algo(key_type);
+    ) -> Result<(PublicKeyMaterial, KeyGenerationTime), Error> {
+        let algo = algo.get_algo(key_type);
         self.generate_key(fp_from_pub, key_type, Some(&algo))
     }
 
+    /// Get public key material from the card.
+    ///
+    /// Note: this fn returns an uninterpreted set of raw values (not an
+    /// OpenPGP key data structure).
+    /// This data from the card is insufficient to create a typical
+    /// full public key.
     pub fn get_pub_key(
         &mut self,
         key_type: KeyType,
-    ) -> Result<PublicKeyMaterial, OpenpgpCardError> {
+    ) -> Result<PublicKeyMaterial, Error> {
         keys::get_pub_key(self, key_type)
     }
 }
