@@ -3,23 +3,22 @@
 
 //! Error types used by this crate.
 //!
-//! [`OpenpgpCardError`] is a wrapper enum for all error types that are used.
+//! [`Error`] is a wrapper enum for all error types that are used.
 //!
 //! The two main classes of errors are:
 //! - [`SmartcardError`], for problems on the reader/smartcard layer
-//! - [`OcErrorStatus`], which models error statuses reported by the OpenPGP
+//! - [`StatusByte`], which models error statuses reported by the OpenPGP
 //! card application
 
-use thiserror::Error;
-
-/// Enum that wraps the different error types that this crate can return
-#[derive(Error, Debug)]
-pub enum OpenpgpCardError {
-    #[error("Error interacting with smartcard {0}")]
+/// Enum wrapper for the different error types of this crate
+#[derive(thiserror::Error, Debug)]
+#[non_exhaustive]
+pub enum Error {
+    #[error("Error interacting with smartcard: {0}")]
     Smartcard(SmartcardError),
 
-    #[error("OpenPGP card error status {0}")]
-    OcStatus(OcErrorStatus),
+    #[error("OpenPGP card error status: {0}")]
+    CardStatus(StatusBytes),
 
     #[error("Command too long ({0} bytes)")]
     CommandTooLong(usize),
@@ -27,25 +26,32 @@ pub enum OpenpgpCardError {
     #[error("Unexpected response length: {0}")]
     ResponseLength(usize),
 
-    #[error("Internal error {0}")]
+    #[error("Internal error: {0}")]
     InternalError(anyhow::Error),
 }
 
-impl From<OcErrorStatus> for OpenpgpCardError {
-    fn from(oce: OcErrorStatus) -> Self {
-        OpenpgpCardError::OcStatus(oce)
+impl From<StatusBytes> for Error {
+    fn from(oce: StatusBytes) -> Self {
+        Error::CardStatus(oce)
     }
 }
 
-impl From<anyhow::Error> for OpenpgpCardError {
+impl From<anyhow::Error> for Error {
     fn from(ae: anyhow::Error) -> Self {
-        OpenpgpCardError::InternalError(ae)
+        Error::InternalError(ae)
     }
 }
 
-/// OpenPGP card "Status Byte" errors
-#[derive(Error, Debug, PartialEq)]
-pub enum OcErrorStatus {
+/// OpenPGP card "Status Bytes" (ok statuses and errors)
+#[derive(thiserror::Error, Debug, PartialEq, Copy, Clone)]
+#[non_exhaustive]
+pub enum StatusBytes {
+    #[error("Command correct")]
+    Ok,
+
+    #[error("Command correct, [{0}] bytes available in response")]
+    OkBytesAvailable(u8),
+
     #[error("Selected file or DO in termination state")]
     TerminationState,
 
@@ -74,7 +80,7 @@ pub enum OcErrorStatus {
     LastCommandOfChainExpected,
 
     #[error("Command chaining not supported")]
-    CommandChainingUnsupported,
+    CommandChainingNotSupported,
 
     #[error("Security status not satisfied")]
     SecurityStatusNotSatisfied,
@@ -116,40 +122,44 @@ pub enum OcErrorStatus {
     UnknownStatus(u8, u8),
 }
 
-impl From<(u8, u8)> for OcErrorStatus {
+impl From<(u8, u8)> for StatusBytes {
     fn from(status: (u8, u8)) -> Self {
         match (status.0, status.1) {
-            (0x62, 0x85) => OcErrorStatus::TerminationState,
+            (0x90, 0x00) => StatusBytes::Ok,
+            (0x61, bytes) => StatusBytes::OkBytesAvailable(bytes),
+
+            (0x62, 0x85) => StatusBytes::TerminationState,
             (0x63, 0xC0..=0xCF) => {
-                OcErrorStatus::PasswordNotChecked(status.1 & 0xf)
+                StatusBytes::PasswordNotChecked(status.1 & 0xf)
             }
-            (0x64, 0x02..=0x80) => OcErrorStatus::TriggeringByCard(status.1),
-            (0x65, 0x01) => OcErrorStatus::MemoryFailure,
-            (0x66, 0x00) => OcErrorStatus::SecurityRelatedIssues,
-            (0x67, 0x00) => OcErrorStatus::WrongLength,
-            (0x68, 0x81) => OcErrorStatus::LogicalChannelNotSupported,
-            (0x68, 0x82) => OcErrorStatus::SecureMessagingNotSupported,
-            (0x68, 0x83) => OcErrorStatus::LastCommandOfChainExpected,
-            (0x68, 0x84) => OcErrorStatus::CommandChainingUnsupported,
-            (0x69, 0x82) => OcErrorStatus::SecurityStatusNotSatisfied,
-            (0x69, 0x83) => OcErrorStatus::AuthenticationMethodBlocked,
-            (0x69, 0x85) => OcErrorStatus::ConditionOfUseNotSatisfied,
-            (0x69, 0x87) => OcErrorStatus::ExpectedSecureMessagingDOsMissing,
-            (0x69, 0x88) => OcErrorStatus::SMDataObjectsIncorrect,
-            (0x6A, 0x80) => OcErrorStatus::IncorrectParametersCommandDataField,
-            (0x6A, 0x82) => OcErrorStatus::FileOrApplicationNotFound,
-            (0x6A, 0x88) => OcErrorStatus::ReferencedDataNotFound,
-            (0x6B, 0x00) => OcErrorStatus::WrongParametersP1P2,
-            (0x6D, 0x00) => OcErrorStatus::INSNotSupported,
-            (0x6E, 0x00) => OcErrorStatus::CLANotSupported,
-            (0x6F, 0x00) => OcErrorStatus::NoPreciseDiagnosis,
-            _ => OcErrorStatus::UnknownStatus(status.0, status.1),
+            (0x64, 0x02..=0x80) => StatusBytes::TriggeringByCard(status.1),
+            (0x65, 0x01) => StatusBytes::MemoryFailure,
+            (0x66, 0x00) => StatusBytes::SecurityRelatedIssues,
+            (0x67, 0x00) => StatusBytes::WrongLength,
+            (0x68, 0x81) => StatusBytes::LogicalChannelNotSupported,
+            (0x68, 0x82) => StatusBytes::SecureMessagingNotSupported,
+            (0x68, 0x83) => StatusBytes::LastCommandOfChainExpected,
+            (0x68, 0x84) => StatusBytes::CommandChainingNotSupported,
+            (0x69, 0x82) => StatusBytes::SecurityStatusNotSatisfied,
+            (0x69, 0x83) => StatusBytes::AuthenticationMethodBlocked,
+            (0x69, 0x85) => StatusBytes::ConditionOfUseNotSatisfied,
+            (0x69, 0x87) => StatusBytes::ExpectedSecureMessagingDOsMissing,
+            (0x69, 0x88) => StatusBytes::SMDataObjectsIncorrect,
+            (0x6A, 0x80) => StatusBytes::IncorrectParametersCommandDataField,
+            (0x6A, 0x82) => StatusBytes::FileOrApplicationNotFound,
+            (0x6A, 0x88) => StatusBytes::ReferencedDataNotFound,
+            (0x6B, 0x00) => StatusBytes::WrongParametersP1P2,
+            (0x6D, 0x00) => StatusBytes::INSNotSupported,
+            (0x6E, 0x00) => StatusBytes::CLANotSupported,
+            (0x6F, 0x00) => StatusBytes::NoPreciseDiagnosis,
+            _ => StatusBytes::UnknownStatus(status.0, status.1),
         }
     }
 }
 
 /// Errors on the smartcard/reader layer
-#[derive(Error, Debug)]
+#[derive(thiserror::Error, Debug)]
+#[non_exhaustive]
 pub enum SmartcardError {
     #[error("Failed to create a pcsc smartcard context {0}")]
     ContextError(String),
