@@ -32,12 +32,13 @@ mod apdu;
 mod card_app;
 pub mod card_do;
 pub mod crypto_data;
-pub mod errors;
+mod errors;
 mod keys;
 mod tlv;
 
 pub use crate::apdu::response::Response;
 pub use crate::card_app::CardApp;
+pub use crate::errors::{Error, SmartcardError, StatusBytes};
 
 /// The CardClient trait defines communication with an OpenPGP card via a
 /// backend implementation (e.g. the pcsc backend in the crate
@@ -106,8 +107,9 @@ impl CardCaps {
     }
 }
 
-/// Enum to identify the Key-slots on an OpenPGP card
+/// Identify a Key slot on an OpenPGP card
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
+#[non_exhaustive]
 pub enum KeyType {
     Signing,
     Decryption,
@@ -158,3 +160,35 @@ impl KeyType {
         }
     }
 }
+
+/// A KeySet binds together a triple of information about each Key on a card
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct KeySet<T> {
+    signature: Option<T>,
+    decryption: Option<T>,
+    authentication: Option<T>,
+}
+
+impl<T> From<(Option<T>, Option<T>, Option<T>)> for KeySet<T> {
+    fn from(tuple: (Option<T>, Option<T>, Option<T>)) -> Self {
+        Self {
+            signature: tuple.0,
+            decryption: tuple.1,
+            authentication: tuple.2,
+        }
+    }
+}
+
+impl<T> KeySet<T> {
+    pub fn signature(&self) -> Option<&T> {
+        self.signature.as_ref()
+    }
+
+    pub fn decryption(&self) -> Option<&T> {
+        self.decryption.as_ref()
+    }
+
+    pub fn authentication(&self) -> Option<&T> {
+        self.authentication.as_ref()
+    }
+}
