@@ -1,19 +1,17 @@
 // SPDX-FileCopyrightText: 2021 Heiko Schaefer <heiko@schaefer.name>
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
+//! Tag in a TLV data structure
+
 use nom::{
     branch, bytes::complete as bytes, combinator, number::complete as number,
     sequence,
 };
 
 #[derive(Clone, Debug, Eq, PartialEq)]
-pub struct Tag(pub Vec<u8>);
+pub struct Tag(Vec<u8>);
 
 impl Tag {
-    pub fn new(t: Vec<u8>) -> Self {
-        Self(t)
-    }
-
     pub fn is_constructed(&self) -> bool {
         if self.0.is_empty() {
             false
@@ -21,6 +19,10 @@ impl Tag {
             self.0[0] & 0x20 != 0
         }
     }
+
+    pub fn get(&self) -> &[u8] {
+        &self.0
+    }
 }
 
 impl From<&[u8]> for Tag {
